@@ -1,5 +1,3 @@
-import datetime
-
 import ckan.authz as authz
 from ckan import logic
 import ckan.plugins.toolkit as toolkit
@@ -14,9 +12,12 @@ def dataset_purge(context, data_dict):
     if not ac["success"]:
         return ac
     # original auth function
+    # (usually, only sysadmins are allowed to purge, so we test against
+    # package_update)
     ao = logic.auth.update.package_update(context, data_dict)
     if not ao["success"]:
         return ao
+
     # get the current package dict
     show_context = {
         'model': context['model'],
@@ -30,7 +31,7 @@ def dataset_purge(context, data_dict):
     state = package_dict.get('state')
     if state != "deleted":
         return {"success": False,
-                "msg": "Only draft datasets can be deleted!"}
+                "msg": "Only deleted datasets can be purged!"}
     return {"success": True}
 
 
@@ -61,8 +62,8 @@ def package_create(context, data_dict):
     ac = login_user(context)
     if not ac["success"]:
         return ac
-    user = context['user']
 
+    user = context['user']
     # If a group is given, check whether the user has the necessary permissions
     check_group = logic.auth.create._check_group_auth(context, data_dict)
     if not check_group:
@@ -70,13 +71,12 @@ def package_create(context, data_dict):
                 'msg': 'User {} not authorized to edit '.format(user)
                        + 'these collections'}
 
-    # If an organization is given are we able to add a dataset to it?
-    data_dict = data_dict or {}
-    org_id = data_dict.get('owner_org')
-    if org_id and not authz.has_user_permission_for_group_or_org(
+    # Are we allowed to add a dataset to the given organization?
+    org_id = logic.get_or_bust(data_dict, 'owner_org')
+    if not authz.has_user_permission_for_group_or_org(
             org_id, user, 'create_dataset'):
         return {'success': False,
-                'msg': 'User {} not authorized to add dataset '.format(user)
+                'msg': 'User {} not authorized to add datasets '.format(user)
                        + 'to circle {}!'.format(org_id)}
 
     return {"success": True}
@@ -92,6 +92,7 @@ def package_delete(context, data_dict):
     ao = logic.auth.update.package_update(context, data_dict)
     if not ao["success"]:
         return ao
+
     # get the current package dict
     show_context = {
         'model': context['model'],
@@ -118,9 +119,7 @@ def package_update(context, data_dict=None):
     ao = logic.auth.update.package_update(context, data_dict)
     if not ao["success"]:
         return ao
-    # nothing to check
-    if data_dict is None:
-        return {'success': True}
+
     # get the current package dict
     show_context = {
         'model': context['model'],
@@ -137,6 +136,14 @@ def package_update(context, data_dict=None):
         if data_dict["license_id"] not in allowed:
             return {'success': False,
                     'msg': 'Cannot switch to more-restrictive license'}
+    # do not allow setting state from "active" to "draft"
+    if package_dict["state"] != "draft" and data_dict.get("state") == "draft":
+        return {'success': False,
+                'msg': 'Changing dataset state to draft not allowed'}
+    # do not allow setting the visibility from public to private
+    if not package_dict["private"] and data_dict.get("private", False):
+        return {'success': False,
+                'msg': 'Changing visibility to private not allowed'}
     # do not allow changing some of the keys
     prohibited_keys = ["name"]
     invalid = {}
@@ -157,44 +164,28 @@ def resource_create(context, data_dict=None):
     ac = login_user(context)
     if not ac["success"]:
         return ac
+    # original auth function
+    ao = logic.auth.create.resource_create(context, data_dict)
+    if not ao["success"]:
+        return ao
 
     if "package_id" in data_dict:
         pkg_dict = logic.get_action('package_show')(
             dict(context, return_type='dict'),
             {'id': data_dict["package_id"]})
 
-        # do not allow adding resources to non-draft datasets after 3h
+        # do not allow adding resources to non-draft datasets
         if pkg_dict["state"] != "draft":
-            # check metadata created
-            dt_str = pkg_dict.get("metadata_created",
-                                  "1999-09-15T08:40:43.450510")
-            dt, _ = dt_str.split(".")
-            created = datetime.datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S")
-            if created + datetime.timedelta(hours=3) < datetime.datetime.now():
-                return {'success': False,
-                        'msg': 'Adding resources to non-draft datasets only '
-                               'allowed within 3 hours after dataset '
-                               'creation!'}
+            return {'success': False,
+                    'msg': 'Adding resources to non-draft datasets not '
+                           'allowed!'}
 
-        # do not allow adding resources that exist already
         if "upload" in data_dict:
-            # check that we got a file
-            try:
-                filename = data_dict["upload"].filename
-            except AttributeError:
-                return {'success': False,
-                        'msg': 'Upload not recognized (got {})!'.format(
-                            data_dict["upload"])}
-
-            # check that id is not set
+            # id must not be set
             if data_dict.get("id", ""):
                 return {'success': False,
                         'msg': 'You are not allowed to set the id!'}
 
-            # check that name is filename
-            if data_dict.get("name", filename) != filename:
-                data_dict["name"] = filename
-
     return {'success': True}
 
 
@@ -203,6 +194,11 @@ def resource_update(context, data_dict=None):
     ac = login_user(context)
     if not ac["success"]:
         return ac
+    # original auth function
+    ao = logic.auth.update.resource_update(context, data_dict)
+    if not ao["success"]:
+        return ao
+
     # get the current resource dict
     show_context = {
         'model': context['model'],
@@ -216,12 +212,8 @@ def resource_update(context, data_dict=None):
     # convert to package id (otherwise the check below might fail)
     convert_package_name_or_id_to_id = toolkit.get_converter(
         'convert_package_name_or_id_to_id')
-    if "package_id" in data_dict:
-        data_dict["package_id"] = convert_package_name_or_id_to_id(
-            data_dict["package_id"], context)
-    else:
-        return {'success': False,
-                'msg': 'No package id specified!'}
+    data_dict["package_id"] = convert_package_name_or_id_to_id(
+            logic.get_or_bust(data_dict, "package_id"), context)
     # only allow updating the description
     allowed_keys = ["description"]
     invalid = {}
